@@ -37,6 +37,7 @@
 typedef struct
 {
   GfMonitorManager       *monitor_manager;
+  GfMonitorsConfig       *config;
   GfLogicalMonitorConfig *logical_monitor_config;
   GfMonitorConfig        *monitor_config;
   GPtrArray              *crtc_infos;
@@ -153,6 +154,37 @@ find_primary_monitor (GfMonitorManager *monitor_manager)
     }
 }
 
+static GfMonitorTransform
+get_monitor_transform (GfMonitorManager *monitor_manager,
+                       GfMonitor        *monitor)
+{
+  GfBackend *backend;
+  GfOrientationManager *orientation_manager;
+
+  if (!gf_monitor_is_laptop_panel (monitor))
+    return GF_MONITOR_TRANSFORM_NORMAL;
+
+  backend = gf_monitor_manager_get_backend (monitor_manager);
+  orientation_manager = gf_backend_get_orientation_manager (backend);
+
+  switch (gf_orientation_manager_get_orientation (orientation_manager))
+    {
+      case GF_ORIENTATION_BOTTOM_UP:
+        return GF_MONITOR_TRANSFORM_180;
+
+      case GF_ORIENTATION_LEFT_UP:
+        return GF_MONITOR_TRANSFORM_90;
+
+      case GF_ORIENTATION_RIGHT_UP:
+        return GF_MONITOR_TRANSFORM_270;
+
+      case GF_ORIENTATION_UNDEFINED:
+      case GF_ORIENTATION_NORMAL:
+      default:
+        return GF_MONITOR_TRANSFORM_NORMAL;
+    }
+}
+
 static GfLogicalMonitorConfig *
 create_preferred_logical_monitor_config (GfMonitorManager           *monitor_manager,
                                          GfMonitor                  *monitor,
@@ -164,6 +196,7 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
   GfMonitorMode *mode;
   int width, height;
   float scale;
+  GfMonitorTransform transform;
   GfMonitorConfig *monitor_config;
   GfLogicalMonitorConfig *logical_monitor_config;
 
@@ -191,6 +224,15 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
     }
 
   monitor_config = gf_monitor_config_new (monitor, mode);
+  transform = get_monitor_transform (monitor_manager, monitor);
+
+  if (gf_monitor_transform_is_rotated (transform))
+    {
+      int temp = width;
+
+      width = height;
+      height = temp;
+    }
 
   logical_monitor_config = g_new0 (GfLogicalMonitorConfig, 1);
   *logical_monitor_config = (GfLogicalMonitorConfig) {
@@ -200,6 +242,7 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
       .width = width,
       .height = height
     },
+    .transform = transform,
     .scale = scale,
     .monitor_configs = g_list_append (NULL, monitor_config)
   };
@@ -381,6 +424,97 @@ create_for_switch_config_builtin (GfMonitorConfigManager *config_manager)
                                  layout_mode, GF_MONITORS_CONFIG_FLAG_NONE);
 }
 
+static GList *
+clone_monitor_config_list (GList *configs_in)
+{
+  GList *configs_out;
+  GList *l;
+
+  configs_out = NULL;
+
+  for (l = configs_in; l != NULL; l = l->next)
+    {
+      GfMonitorConfig *config_in;
+      GfMonitorConfig *config_out;
+
+      config_in = l->data;
+
+      config_out = g_new0 (GfMonitorConfig, 1);
+      *config_out = (GfMonitorConfig) {
+        .monitor_spec = gf_monitor_spec_clone (config_in->monitor_spec),
+        .mode_spec = g_memdup (config_in->mode_spec, sizeof (GfMonitorModeSpec)),
+        .enable_underscanning = config_in->enable_underscanning
+      };
+
+      configs_out = g_list_append (configs_out, config_out);
+    }
+
+  return configs_out;
+}
+
+static GList *
+clone_logical_monitor_config_list (GList *configs_in)
+{
+  GList *configs_out;
+  GList *l;
+
+  configs_out = NULL;
+
+  for (l = configs_in; l != NULL; l = l->next)
+    {
+      GfLogicalMonitorConfig *config_in;
+      GfLogicalMonitorConfig *config_out;
+      GList *config_list;
+
+      config_in = l->data;
+
+      config_out = g_memdup (config_in, sizeof (GfLogicalMonitorConfig));
+
+      config_list = clone_monitor_config_list (config_in->monitor_configs);
+      config_out->monitor_configs = config_list;
+
+      configs_out = g_list_append (configs_out, config_out);
+    }
+
+  return configs_out;
+}
+
+static GfLogicalMonitorConfig *
+find_logical_config_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
+                                                  GList                  *logical_monitor_configs)
+{
+  GfMonitor *panel;
+  GList *l;
+
+  panel = gf_monitor_manager_get_laptop_panel (config_manager->monitor_manager);
+
+  if (panel == NULL || !gf_monitor_is_active (panel))
+    return NULL;
+
+  for (l = logical_monitor_configs; l; l = l->next)
+    {
+      GfLogicalMonitorConfig *logical_monitor_config;
+      GfMonitorConfig *monitor_config;
+
+      logical_monitor_config = l->data;
+
+      /*
+       * We only want to return the config for the panel if it is
+       * configured on its own, so we skip configs which contain clones.
+       */
+      if (g_list_length (logical_monitor_config->monitor_configs) != 1)
+        continue;
+
+      monitor_config = logical_monitor_config->monitor_configs->data;
+      if (gf_monitor_spec_equals (gf_monitor_get_spec (panel),
+                                  monitor_config->monitor_spec))
+        return logical_monitor_config;
+    }
+
+  return NULL;
+}
+
+
 static GfMonitorsConfig *
 create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
                                      gboolean                rotate,
@@ -390,20 +524,19 @@ create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
   GfLogicalMonitorConfig *logical_monitor_config;
   GfLogicalMonitorConfig *current_logical_monitor_config;
   GList *logical_monitor_configs;
+  GList *current_configs;
   GfLogicalMonitorLayoutMode layout_mode;
-  GfMonitorConfig *monitor_config;
-  GfMonitorConfig *current_monitor_config;
-
-  if (!gf_monitor_manager_get_is_builtin_display_on (config_manager->monitor_manager))
-    return NULL;
+  GList *current_monitor_configs;
 
   if (!config_manager->current_config)
     return NULL;
 
-  if (g_list_length (config_manager->current_config->logical_monitor_configs) != 1)
-    return NULL;
+  current_configs = config_manager->current_config->logical_monitor_configs;
+  current_logical_monitor_config = find_logical_config_for_builtin_display_rotation (config_manager,
+                                                                                     current_configs);
 
-  current_logical_monitor_config = config_manager->current_config->logical_monitor_configs->data;
+  if (!current_logical_monitor_config)
+    return NULL;
 
   if (rotate)
     transform = (current_logical_monitor_config->transform + 1) % GF_MONITOR_TRANSFORM_FLIPPED;
@@ -425,20 +558,11 @@ create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
   if (current_logical_monitor_config->transform == transform)
     return NULL;
 
-  if (g_list_length (current_logical_monitor_config->monitor_configs) != 1)
-    return NULL;
+  current_monitor_configs = config_manager->current_config->logical_monitor_configs;
+  logical_monitor_configs = clone_logical_monitor_config_list (current_monitor_configs);
 
-  current_monitor_config = current_logical_monitor_config->monitor_configs->data;
-
-  monitor_config = g_new0 (GfMonitorConfig, 1);
-  *monitor_config = (GfMonitorConfig) {
-    .monitor_spec = gf_monitor_spec_clone (current_monitor_config->monitor_spec),
-    .mode_spec = g_memdup (current_monitor_config->mode_spec, sizeof (GfMonitorModeSpec)),
-    .enable_underscanning = current_monitor_config->enable_underscanning
-  };
-
-  logical_monitor_config = g_memdup (current_logical_monitor_config, sizeof (GfLogicalMonitorConfig));
-  logical_monitor_config->monitor_configs = g_list_append (NULL, monitor_config);
+  logical_monitor_config = find_logical_config_for_builtin_display_rotation (config_manager,
+                                                                             logical_monitor_configs);
   logical_monitor_config->transform = transform;
 
   if (gf_monitor_transform_is_rotated (current_logical_monitor_config->transform) !=
@@ -450,7 +574,6 @@ create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
       logical_monitor_config->layout.height = temp;
     }
 
-  logical_monitor_configs = g_list_append (NULL, logical_monitor_config);
   layout_mode = config_manager->current_config->layout_mode;
 
   return gf_monitors_config_new (monitor_manager, logical_monitor_configs,
@@ -543,6 +666,11 @@ assign_monitor_crtc (GfMonitor          *monitor,
   GfMonitorTransform transform;
   GfMonitorTransform crtc_transform;
   int crtc_x, crtc_y;
+  float x_offset, y_offset;
+  float scale;
+  float width, height;
+  GfCrtcMode *crtc_mode;
+  GfRectangle crtc_layout;
   GfCrtcInfo *crtc_info;
   GfOutputInfo *output_info;
   GfMonitorConfig *first_monitor_config;
@@ -572,29 +700,49 @@ assign_monitor_crtc (GfMonitor          *monitor,
   gf_monitor_calculate_crtc_pos (monitor, mode, output, crtc_transform,
                                  &crtc_x, &crtc_y);
 
+  x_offset = data->logical_monitor_config->layout.x;
+  y_offset = data->logical_monitor_config->layout.y;
+
+  switch (data->config->layout_mode)
+    {
+      case GF_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
+        scale = data->logical_monitor_config->scale;
+        break;
+
+      case GF_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
+      default:
+        scale = 1.0;
+        break;
+    }
+
+  crtc_mode = monitor_crtc_mode->crtc_mode;
+
+  if (gf_monitor_transform_is_rotated (crtc_transform))
+    {
+      width = crtc_mode->height / scale;
+      height = crtc_mode->width / scale;
+    }
+  else
+    {
+      width = crtc_mode->width / scale;
+      height = crtc_mode->height / scale;
+    }
+
+  crtc_layout.x = (int) roundf (x_offset + (crtc_x / scale));
+  crtc_layout.y = (int) roundf (y_offset + (crtc_y / scale));
+  crtc_layout.width = (int) roundf (width);
+  crtc_layout.height = (int) roundf (height);
+
   crtc_info = g_slice_new0 (GfCrtcInfo);
   *crtc_info = (GfCrtcInfo) {
     .crtc = crtc,
-    .mode = monitor_crtc_mode->crtc_mode,
-    .x = crtc_x,
-    .y = crtc_y,
+    .mode = crtc_mode,
+    .layout = crtc_layout,
     .transform = crtc_transform,
     .outputs = g_ptr_array_new ()
   };
   g_ptr_array_add (crtc_info->outputs, output);
 
-  /*
-   * Currently, GfCrtcInfo are deliberately offset incorrectly to carry over
-   * logical monitor location inside the GfCrtc struct, when in fact this
-   * depends on the framebuffer configuration. This will eventually be negated
-   * when setting the actual KMS mode.
-   *
-   * TODO: Remove this hack when we don't need to rely on GfCrtc to pass
-   * logical monitor state.
-   */
-  crtc_info->x += data->logical_monitor_config->layout.x;
-  crtc_info->y += data->logical_monitor_config->layout.y;
-
   /*
    * Only one output can be marked as primary (due to Xrandr limitation),
    * so only mark the main output of the first monitor in the logical monitor
@@ -629,6 +777,7 @@ assign_monitor_crtc (GfMonitor          *monitor,
 
 static gboolean
 assign_monitor_crtcs (GfMonitorManager        *manager,
+                      GfMonitorsConfig        *config,
                       GfLogicalMonitorConfig  *logical_monitor_config,
                       GfMonitorConfig         *monitor_config,
                       GPtrArray               *crtc_infos,
@@ -666,6 +815,7 @@ assign_monitor_crtcs (GfMonitorManager        *manager,
 
   data = (MonitorAssignmentData) {
     .monitor_manager = manager,
+    .config = config,
     .logical_monitor_config = logical_monitor_config,
     .monitor_config = monitor_config,
     .crtc_infos = crtc_infos,
@@ -683,6 +833,7 @@ assign_monitor_crtcs (GfMonitorManager        *manager,
 
 static gboolean
 assign_logical_monitor_crtcs (GfMonitorManager        *manager,
+                              GfMonitorsConfig        *config,
                               GfLogicalMonitorConfig  *logical_monitor_config,
                               GPtrArray               *crtc_infos,
                               GPtrArray               *output_infos,
@@ -696,6 +847,7 @@ assign_logical_monitor_crtcs (GfMonitorManager        *manager,
       GfMonitorConfig *monitor_config = l->data;
 
       if (!assign_monitor_crtcs (manager,
+                                 config,
                                  logical_monitor_config,
                                  monitor_config,
                                  crtc_infos, output_infos,
@@ -844,9 +996,13 @@ gf_monitor_config_manager_assign (GfMonitorManager  *manager,
     {
       GfLogicalMonitorConfig *logical_monitor_config = l->data;
 
-      if (!assign_logical_monitor_crtcs (manager, logical_monitor_config,
-                                         crtc_infos, output_infos,
-                                         reserved_crtcs, error))
+      if (!assign_logical_monitor_crtcs (manager,
+                                         config,
+                                         logical_monitor_config,
+                                         crtc_infos,
+                                         output_infos,
+                                         reserved_crtcs,
+                                         error))
         {
           g_ptr_array_free (crtc_infos, TRUE);
           g_ptr_array_free (output_infos, TRUE);
