@@ -34,6 +34,7 @@
 #include "gf-monitor-manager-enums-private.h"
 #include "gf-monitor-manager-types-private.h"
 #include "gf-monitor-manager.h"
+#include "gf-monitor-transform.h"
 
 G_BEGIN_DECLS
 
@@ -54,8 +55,6 @@ struct _GfMonitorManager
 
   guint                        serial;
 
-  GfPowerSave                  power_save_mode;
-
   GfLogicalMonitorLayoutMode   layout_mode;
 
   gint                         screen_width;
@@ -86,6 +85,8 @@ typedef struct
 
   gboolean                     (* is_lid_closed)                (GfMonitorManager            *manager);
 
+  void                         (* read_current_state)           (GfMonitorManager            *manager);
+
   void                         (* ensure_initial_config)        (GfMonitorManager            *manager);
 
   gboolean                     (* apply_monitors_config)        (GfMonitorManager            *manager,
@@ -226,17 +227,10 @@ GfMonitorConfigManager     *gf_monitor_manager_get_config_manager           (GfM
 char                       *gf_monitor_manager_get_vendor_name              (GfMonitorManager            *manager,
                                                                              const char                  *vendor);
 
-static inline gboolean
-gf_monitor_transform_is_rotated (GfMonitorTransform transform)
-{
-  return (transform % 2);
-}
+GfPowerSave                 gf_monitor_manager_get_power_save_mode          (GfMonitorManager            *manager);
 
-static inline gboolean
-gf_monitor_transform_is_flipped (GfMonitorTransform transform)
-{
-  return (transform >= GF_MONITOR_TRANSFORM_FLIPPED);
-}
+void                        gf_monitor_manager_power_save_mode_changed      (GfMonitorManager            *manager,
+                                                                             GfPowerSave                  mode);
 
 G_END_DECLS
 
