@@ -211,39 +211,14 @@ is_crtc_assignment_changed (GfCrtc      *crtc,
   for (i = 0; i < n_crtc_infos; i++)
     {
       GfCrtcInfo *crtc_info = crtc_infos[i];
-      guint j;
 
       if (crtc_info->crtc != crtc)
         continue;
 
-      if (crtc->current_mode != crtc_info->mode)
-        return TRUE;
-
-      if (crtc->rect.x != crtc_info->x)
-        return TRUE;
-
-      if (crtc->rect.y != crtc_info->y)
-        return TRUE;
-
-      if (crtc->transform != crtc_info->transform)
-        return TRUE;
-
-      for (j = 0; j < crtc_info->outputs->len; j++)
-        {
-          GfOutput *output;
-          GfCrtc *assigned_crtc;
-
-          output = ((GfOutput**) crtc_info->outputs->pdata)[j];
-          assigned_crtc = gf_output_get_assigned_crtc (output);
-
-          if (assigned_crtc != crtc)
-            return TRUE;
-        }
-
-      return FALSE;
+      return gf_crtc_xrandr_is_assignment_changed (crtc, crtc_info);
     }
 
-  return crtc->current_mode != NULL;
+  return !!gf_crtc_xrandr_get_current_mode (crtc);
 }
 
 static gboolean
@@ -416,16 +391,8 @@ apply_crtc_assignments (GfMonitorManager  *manager,
       if (crtc_info->mode == NULL)
         continue;
 
-      if (gf_monitor_transform_is_rotated (crtc_info->transform))
-        {
-          width = MAX (width, crtc_info->x + crtc_info->mode->height);
-          height = MAX (height, crtc_info->y + crtc_info->mode->width);
-        }
-      else
-        {
-          width = MAX (width, crtc_info->x + crtc_info->mode->width);
-          height = MAX (height, crtc_info->y + crtc_info->mode->height);
-        }
+      width = MAX (width, crtc_info->layout.x + crtc_info->layout.width);
+      height = MAX (height, crtc_info->layout.y + crtc_info->layout.height);
     }
 
   /* Second disable all newly disabled CRTCs, or CRTCs that in the previous
@@ -437,10 +404,17 @@ apply_crtc_assignments (GfMonitorManager  *manager,
     {
       GfCrtcInfo *crtc_info = crtcs[i];
       GfCrtc *crtc = crtc_info->crtc;
+      GfCrtcConfig *crtc_config;
+      int x2, y2;
 
-      if (crtc_info->mode == NULL ||
-          crtc->rect.x + crtc->rect.width > width ||
-          crtc->rect.y + crtc->rect.height > height)
+      crtc_config = crtc->config;
+      if (crtc_config == NULL)
+        continue;
+
+      x2 = crtc_config->layout.x + crtc_config->layout.width;
+      y2 = crtc_config->layout.y + crtc_config->layout.height;
+
+      if (crtc_info->mode == NULL || x2 > width || y2 > height)
         {
           xrandr_set_crtc_config (xrandr,
                                   crtc,
@@ -451,11 +425,7 @@ apply_crtc_assignments (GfMonitorManager  *manager,
                                   XCB_RANDR_ROTATION_ROTATE_0,
                                   NULL, 0);
 
-          crtc->rect.x = 0;
-          crtc->rect.y = 0;
-          crtc->rect.width = 0;
-          crtc->rect.height = 0;
-          crtc->current_mode = NULL;
+          gf_crtc_unset_config (crtc);
         }
     }
 
@@ -470,7 +440,7 @@ apply_crtc_assignments (GfMonitorManager  *manager,
           continue;
         }
 
-      if (crtc->current_mode == NULL)
+      if (!crtc->config)
         continue;
 
       xrandr_set_crtc_config (xrandr,
@@ -482,11 +452,7 @@ apply_crtc_assignments (GfMonitorManager  *manager,
                               XCB_RANDR_ROTATION_ROTATE_0,
                               NULL, 0);
 
-      crtc->rect.x = 0;
-      crtc->rect.y = 0;
-      crtc->rect.width = 0;
-      crtc->rect.height = 0;
-      crtc->current_mode = NULL;
+      gf_crtc_unset_config (crtc);
     }
 
   g_assert (width > 0 && height > 0);
@@ -536,7 +502,8 @@ apply_crtc_assignments (GfMonitorManager  *manager,
                                        save_timestamp,
                                        (xcb_randr_crtc_t) crtc->crtc_id,
                                        XCB_CURRENT_TIME,
-                                       crtc_info->x, crtc_info->y,
+                                       crtc_info->layout.x,
+                                       crtc_info->layout.y,
                                        (xcb_randr_mode_t) mode->mode_id,
                                        rotation,
                                        output_ids, n_output_ids))
@@ -544,29 +511,17 @@ apply_crtc_assignments (GfMonitorManager  *manager,
               g_warning ("Configuring CRTC %d with mode %d (%d x %d @ %f) at position %d, %d and transform %u failed\n",
                          (guint) (crtc->crtc_id), (guint) (mode->mode_id),
                          mode->width, mode->height, (gdouble) mode->refresh_rate,
-                         crtc_info->x, crtc_info->y, crtc_info->transform);
+                         crtc_info->layout.x, crtc_info->layout.y,
+                         crtc_info->transform);
 
               g_free (output_ids);
               continue;
             }
 
-          if (gf_monitor_transform_is_rotated (crtc_info->transform))
-            {
-              width = mode->height;
-              height = mode->width;
-            }
-          else
-            {
-              width = mode->width;
-              height = mode->height;
-            }
-
-          crtc->rect.x = crtc_info->x;
-          crtc->rect.y = crtc_info->y;
-          crtc->rect.width = width;
-          crtc->rect.height = height;
-          crtc->current_mode = mode;
-          crtc->transform = crtc_info->transform;
+          gf_crtc_set_config (crtc,
+                              &crtc_info->layout,
+                              mode,
+                              crtc_info->transform);
 
           g_free (output_ids);
         }
@@ -776,6 +731,55 @@ gf_monitor_manager_xrandr_read_edid (GfMonitorManager *manager,
   return gf_output_xrandr_read_edid (output);
 }
 
+static void
+gf_monitor_manager_xrandr_read_current_state (GfMonitorManager *manager)
+{
+  GfMonitorManagerXrandr *self;
+  CARD16 dpms_state;
+  BOOL dpms_enabled;
+  GfPowerSave power_save_mode;
+  GfMonitorManagerClass *parent_class;
+
+  self = GF_MONITOR_MANAGER_XRANDR (manager);
+
+  if (DPMSCapable (self->xdisplay) &&
+      DPMSInfo (self->xdisplay, &dpms_state, &dpms_enabled) &&
+      dpms_enabled)
+    {
+      switch (dpms_state)
+        {
+          case DPMSModeOn:
+            power_save_mode = GF_POWER_SAVE_ON;
+            break;
+
+          case DPMSModeStandby:
+            power_save_mode = GF_POWER_SAVE_STANDBY;
+            break;
+
+          case DPMSModeSuspend:
+            power_save_mode = GF_POWER_SAVE_SUSPEND;
+            break;
+
+          case DPMSModeOff:
+            power_save_mode = GF_POWER_SAVE_OFF;
+            break;
+
+          default:
+            power_save_mode = GF_POWER_SAVE_UNSUPPORTED;
+            break;
+        }
+    }
+  else
+    {
+      power_save_mode = GF_POWER_SAVE_UNSUPPORTED;
+    }
+
+  gf_monitor_manager_power_save_mode_changed (manager, power_save_mode);
+
+  parent_class = GF_MONITOR_MANAGER_CLASS (gf_monitor_manager_xrandr_parent_class);
+  parent_class->read_current_state (manager);
+}
+
 static void
 gf_monitor_manager_xrandr_ensure_initial_config (GfMonitorManager *manager)
 {
@@ -1101,6 +1105,7 @@ gf_monitor_manager_xrandr_class_init (GfMonitorManagerXrandrClass *xrandr_class)
   object_class->finalize = gf_monitor_manager_xrandr_finalize;
 
   manager_class->read_edid = gf_monitor_manager_xrandr_read_edid;
+  manager_class->read_current_state = gf_monitor_manager_xrandr_read_current_state;
   manager_class->ensure_initial_config = gf_monitor_manager_xrandr_ensure_initial_config;
   manager_class->apply_monitors_config = gf_monitor_manager_xrandr_apply_monitors_config;
   manager_class->set_power_save_mode = gf_monitor_manager_xrandr_set_power_save_mode;
