@@ -45,13 +45,15 @@
 
 typedef struct
 {
-  GfBackend *backend;
+  GfBackend   *backend;
 
-  gboolean   in_init;
+  gboolean     in_init;
 
-  guint      bus_name_id;
+  guint        bus_name_id;
 
-  guint      persistent_timeout_id;
+  guint        persistent_timeout_id;
+
+  GfPowerSave  power_save_mode;
 } GfMonitorManagerPrivate;
 
 typedef gboolean (* MonitorMatchFunc) (GfMonitor *monitor);
@@ -133,9 +135,11 @@ power_save_mode_changed (GfMonitorManager *manager,
                          GParamSpec       *pspec,
                          gpointer          user_data)
 {
+  GfMonitorManagerPrivate *priv;
   GfMonitorManagerClass *manager_class;
   gint mode;
 
+  priv = gf_monitor_manager_get_instance_private (manager);
   manager_class = GF_MONITOR_MANAGER_GET_CLASS (manager);
   mode = gf_dbus_display_config_get_power_save_mode (manager->display_config);
 
@@ -143,7 +147,7 @@ power_save_mode_changed (GfMonitorManager *manager,
     return;
 
   /* If DPMS is unsupported, force the property back. */
-  if (manager->power_save_mode == GF_POWER_SAVE_UNSUPPORTED)
+  if (priv->power_save_mode == GF_POWER_SAVE_UNSUPPORTED)
     {
       gf_dbus_display_config_set_power_save_mode (manager->display_config,
                                                   GF_POWER_SAVE_UNSUPPORTED);
@@ -153,9 +157,7 @@ power_save_mode_changed (GfMonitorManager *manager,
   if (manager_class->set_power_save_mode)
     manager_class->set_power_save_mode (manager, mode);
 
-  manager->power_save_mode = mode;
-
-  g_signal_emit (manager, manager_signals[POWER_SAVE_MODE_CHANGED], 0);
+  gf_monitor_manager_power_save_mode_changed (manager, mode);
 }
 
 static void
@@ -1080,29 +1082,47 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
     {
       GfCrtc *crtc = l->data;
       GVariantBuilder transforms;
-      gint current_mode_index;
+      GfCrtcConfig *crtc_config;
 
       g_variant_builder_init (&transforms, G_VARIANT_TYPE ("au"));
       for (j = 0; j <= GF_MONITOR_TRANSFORM_FLIPPED_270; j++)
         if (crtc->all_transforms & (1 << j))
           g_variant_builder_add (&transforms, "u", j);
 
-      if (crtc->current_mode)
-        current_mode_index = g_list_index (combined_modes, crtc->current_mode);
-      else
-        current_mode_index = -1;
+      crtc_config = crtc->config;
 
-      g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
-                             i, /* ID */
-                             (gint64) crtc->crtc_id,
-                             (gint) crtc->rect.x,
-                             (gint) crtc->rect.y,
-                             (gint) crtc->rect.width,
-                             (gint) crtc->rect.height,
-                             current_mode_index,
-                             (guint32) crtc->transform,
-                             &transforms,
-                             NULL /* properties */);
+      if (crtc_config != NULL)
+        {
+          int current_mode_index;
+
+          current_mode_index = g_list_index (combined_modes, crtc_config->mode);
+
+          g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
+                                 i, /* ID */
+                                 (int64_t) crtc->crtc_id,
+                                 crtc_config->layout.x,
+                                 crtc_config->layout.y,
+                                 crtc_config->layout.width,
+                                 crtc_config->layout.height,
+                                 current_mode_index,
+                                 (uint32_t) crtc_config->transform,
+                                 &transforms,
+                                 NULL /* properties */);
+        }
+      else
+        {
+          g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
+                                 i, /* ID */
+                                 0,
+                                 0,
+                                 0,
+                                 0,
+                                 0,
+                                 -1,
+                                 (uint32_t) GF_MONITOR_TRANSFORM_NORMAL,
+                                 &transforms,
+                                 NULL /* properties */);
+        }
     }
 
   for (l = combined_outputs, i = 0; l; l = l->next, i++)
@@ -1868,13 +1888,24 @@ bus_acquired_cb (GDBusConnection *connection,
 {
   GfMonitorManager *manager;
   GDBusInterfaceSkeleton *skeleton;
+  GError *error;
+  gboolean exported;
 
   manager = GF_MONITOR_MANAGER (user_data);
   skeleton = G_DBUS_INTERFACE_SKELETON (manager->display_config);
 
-  g_dbus_interface_skeleton_export (skeleton, connection,
-                                    "/org/gnome/Mutter/DisplayConfig",
-                                    NULL);
+  error = NULL;
+  exported = g_dbus_interface_skeleton_export (skeleton,
+                                               connection,
+                                               "/org/gnome/Mutter/DisplayConfig",
+                                               &error);
+
+  if (!exported)
+    {
+      g_warning ("%s", error->message);
+      g_error_free (error);
+      return;
+    }
 }
 
 static void
@@ -1907,6 +1938,28 @@ gf_monitor_manager_real_is_lid_closed (GfMonitorManager *manager)
   return manager->lid_is_closed;
 }
 
+static void
+gf_monitor_manager_real_read_current_state (GfMonitorManager *manager)
+{
+  GList *l;
+
+  manager->serial++;
+
+  for (l = manager->gpus; l; l = l->next)
+    {
+      GfGpu *gpu = l->data;
+      GError *error = NULL;
+
+      if (!gf_gpu_read_current (gpu, &error))
+        {
+          g_warning ("Failed to read current KMS state: %s", error->message);
+          g_clear_error (&error);
+        }
+    }
+
+  rebuild_monitors (manager);
+}
+
 static void
 lid_is_closed_changed (UpClient   *client,
                        GParamSpec *pspec,
@@ -2119,6 +2172,7 @@ gf_monitor_manager_class_init (GfMonitorManagerClass *manager_class)
 
   manager_class->read_edid = gf_monitor_manager_real_read_edid;
   manager_class->is_lid_closed = gf_monitor_manager_real_is_lid_closed;
+  manager_class->read_current_state = gf_monitor_manager_real_read_current_state;
 
   gf_monitor_manager_install_properties (object_class);
   gf_monitor_manager_install_signals (object_class);
@@ -2259,23 +2313,7 @@ gf_monitor_manager_has_hotplug_mode_update (GfMonitorManager *manager)
 void
 gf_monitor_manager_read_current_state (GfMonitorManager *manager)
 {
-  GList *l;
-
-  manager->serial++;
-
-  for (l = manager->gpus; l; l = l->next)
-    {
-      GfGpu *gpu = l->data;
-      GError *error = NULL;
-
-      if (!gf_gpu_read_current (gpu, &error))
-        {
-          g_warning ("Failed to read current KMS state: %s", error->message);
-          g_clear_error (&error);
-        }
-    }
-
-  rebuild_monitors (manager);
+  return GF_MONITOR_MANAGER_GET_CLASS (manager)->read_current_state (manager);
 }
 
 void
@@ -2460,9 +2498,14 @@ gf_monitor_manager_ensure_configured (GfMonitorManager *manager)
 done:
   if (!config)
     {
-      gf_monitor_manager_apply_monitors_config (manager, NULL,
-                                                fallback_method,
-                                                &error);
+      if (!gf_monitor_manager_apply_monitors_config (manager, NULL,
+                                                     fallback_method,
+                                                     &error))
+        {
+          g_warning ("%s", error->message);
+          g_clear_error (&error);
+        }
+
       return NULL;
     }
 
@@ -2595,6 +2638,31 @@ gf_monitor_manager_get_vendor_name (GfMonitorManager *manager,
   return gnome_pnp_ids_get_pnp_id (manager->pnp_ids, vendor);
 }
 
+GfPowerSave
+gf_monitor_manager_get_power_save_mode (GfMonitorManager *manager)
+{
+  GfMonitorManagerPrivate *priv;
+
+  priv = gf_monitor_manager_get_instance_private (manager);
+
+  return priv->power_save_mode;
+}
+
+void
+gf_monitor_manager_power_save_mode_changed (GfMonitorManager *manager,
+                                            GfPowerSave       mode)
+{
+  GfMonitorManagerPrivate *priv;
+
+  priv = gf_monitor_manager_get_instance_private (manager);
+
+  if (priv->power_save_mode == mode)
+    return;
+
+  priv->power_save_mode = mode;
+  g_signal_emit (manager, manager_signals[POWER_SAVE_MODE_CHANGED], 0);
+}
+
 gint
 gf_monitor_manager_get_monitor_for_connector (GfMonitorManager *manager,
                                               const gchar      *connector)
