@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Alberts Muktupāvels
+ * Copyright (C) 2014-2020 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include "liba11y-keyboard/gf-a11y-keyboard.h"
 #include "libaudio-device-selection/gf-audio-device-selection.h"
 #include "libautomount-manager/gsd-automount-manager.h"
-#include "libbluetooth-applet/gf-bluetooth-applet.h"
 #include "libclipboard/gf-clipboard.h"
 #include "libdesktop/gf-desktop.h"
 #include "libend-session-dialog/gf-end-session-dialog.h"
@@ -35,13 +34,11 @@
 #include "libinput-sources/gf-input-sources.h"
 #include "libnotifications/gf-notifications.h"
 #include "libpolkit/flashback-polkit.h"
-#include "libpower-applet/gf-power-applet.h"
 #include "libroot-background/gf-root-background.h"
 #include "libscreencast/gf-screencast.h"
 #include "libscreensaver/gf-screensaver.h"
 #include "libscreenshot/gf-screenshot.h"
 #include "libshell/flashback-shell.h"
-#include "libsound-applet/gf-sound-applet.h"
 #include "libstatus-notifier-watcher/gf-status-notifier-watcher.h"
 
 struct _GfApplication
@@ -62,19 +59,16 @@ struct _GfApplication
   FlashbackShell          *shell;
   GfA11yKeyboard          *a11y_keyboard;
   GfAudioDeviceSelection  *audio_device_selection;
-  GfBluetoothApplet       *bluetooth;
   GfClipboard             *clipboard;
   GfDesktop               *desktop;
   GfEndSessionDialog      *dialog;
   GfInputSettings         *input_settings;
   GfInputSources          *input_sources;
   GfNotifications         *notifications;
-  GfPowerApplet           *power;
   GfRootBackground        *root_background;
   GfScreencast            *screencast;
   GfScreensaver           *screensaver;
   GfScreenshot            *screenshot;
-  GfSoundApplet           *sound;
   GfStatusNotifierWatcher *status_notifier_watcher;
 
   GtkWidget               *display_change_dialog;
@@ -96,6 +90,9 @@ static GfSupportedTheme supported_themes[] =
     { "Adwaita-dark", "Adwaita", "dark", FALSE },
     { "HighContrast", "HighContrast", NULL, FALSE },
     { "HighContrastInverse", "HighContrast", "inverse", FALSE },
+    { "Yaru", "Yaru", NULL, FALSE },
+    { "Yaru-light", "Yaru", NULL, FALSE },
+    { "Yaru-dark", "Yaru", NULL, FALSE },
     { NULL, NULL, FALSE, FALSE }
   };
 
@@ -220,19 +217,16 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (shell, "shell", flashback_shell_new)
   SETTING_CHANGED (a11y_keyboard, "a11y-keyboard", gf_a11y_keyboard_new)
   SETTING_CHANGED (audio_device_selection, "audio-device-selection", gf_audio_device_selection_new)
-  SETTING_CHANGED (bluetooth, "bluetooth-applet", gf_bluetooth_applet_new)
   SETTING_CHANGED (desktop, "desktop", gf_desktop_new)
   SETTING_CHANGED (clipboard, "clipboard", gf_clipboard_new)
   SETTING_CHANGED (dialog, "end-session-dialog", gf_end_session_dialog_new)
   SETTING_CHANGED (input_settings, "input-settings", gf_input_settings_new)
   SETTING_CHANGED (input_sources, "input-sources", gf_input_sources_new)
   SETTING_CHANGED (notifications, "notifications", gf_notifications_new)
-  SETTING_CHANGED (power, "power-applet", gf_power_applet_new)
   SETTING_CHANGED (root_background, "root-background", gf_root_background_new)
   SETTING_CHANGED (screencast, "screencast", gf_screencast_new)
   SETTING_CHANGED (screensaver, "screensaver", gf_screensaver_new)
   SETTING_CHANGED (screenshot, "screenshot", gf_screenshot_new)
-  SETTING_CHANGED (sound, "sound-applet", gf_sound_applet_new)
   SETTING_CHANGED (status_notifier_watcher, "status-notifier-watcher", gf_status_notifier_watcher_new)
 
 #undef SETTING_CHANGED
@@ -241,6 +235,12 @@ settings_changed (GSettings   *settings,
     gf_input_settings_set_monitor_manager (application->input_settings,
                                            monitor_manager);
 
+  if (application->screensaver)
+    {
+      gf_screensaver_set_input_sources (application->screensaver,
+                                        application->input_sources);
+    }
+
   if (application->shell)
     flashback_shell_set_monitor_manager (application->shell, monitor_manager);
 }
@@ -268,19 +268,16 @@ gf_application_dispose (GObject *object)
   g_clear_object (&application->shell);
   g_clear_object (&application->a11y_keyboard);
   g_clear_object (&application->audio_device_selection);
-  g_clear_object (&application->bluetooth);
   g_clear_object (&application->desktop);
   g_clear_object (&application->clipboard);
   g_clear_object (&application->dialog);
   g_clear_object (&application->input_settings);
   g_clear_object (&application->input_sources);
   g_clear_object (&application->notifications);
-  g_clear_object (&application->power);
   g_clear_object (&application->root_background);
   g_clear_object (&application->screencast);
   g_clear_object (&application->screenshot);
   g_clear_object (&application->screensaver);
-  g_clear_object (&application->sound);
   g_clear_object (&application->status_notifier_watcher);
 
   g_clear_pointer (&application->display_change_dialog, gtk_widget_destroy);
